/***** Start of precommondefs.h *****/

/* This is extracted from pyconfig.h from CPython.  It sets the macros
   that affect the features we get from system include files.
   It must not #include anything. */

#ifndef __PYPY_PRECOMMONDEFS_H
#define __PYPY_PRECOMMONDEFS_H


/* Define on Darwin to activate all library features */
#define _DARWIN_C_SOURCE 1
/* This must be set to 64 on some systems to enable large file support. */
#define _FILE_OFFSET_BITS 64
/* Define on Linux to activate all library features */
#define _GNU_SOURCE 1
/* This must be defined on some systems to enable large file support. */
#define _LARGEFILE_SOURCE 1
/* Define on NetBSD to activate all library features */
#define _NETBSD_SOURCE 1
/* Define to activate features from IEEE Stds 1003.1-2008 */
#ifndef _POSIX_C_SOURCE
#  define _POSIX_C_SOURCE 200809L
#endif
/* Define on FreeBSD to activate all library features */
#define __BSD_VISIBLE 1
#define __XSI_VISIBLE 700
/* Windows: winsock/winsock2 mess */
#define WIN32_LEAN_AND_MEAN
#ifdef _WIN64
   typedef          __int64 Signed;
   typedef unsigned __int64 Unsigned;
#  define SIGNED_MIN LLONG_MIN
#else
   typedef          long Signed;
   typedef unsigned long Unsigned;
#  define SIGNED_MIN LONG_MIN
#endif

#if !defined(RPY_ASSERT) && !defined(RPY_LL_ASSERT)
#  define NDEBUG
#endif


/* All functions and global variables declared anywhere should use
   one of the following attributes:

   RPY_EXPORTED:  the symbol is exported out of libpypy-c.so.

   RPY_EXTERN:    the symbol is not exported out of libpypy-c.so, but
                  otherwise works like 'extern' by being available to
                  other C sources.

   static:        as usual, this means the symbol is local to this C file.

   Don't use _RPY_HIDDEN directly.  For tests involving building a custom
   .so, translator/tool/cbuild.py overrides RPY_EXTERN so that it becomes
   equal to RPY_EXPORTED.

   Any function or global variable declared with no attribute at all is
   a bug; please report or fix it.
*/
#ifdef __GNUC__
#  define RPY_EXPORTED extern __attribute__((visibility("default")))
#  define _RPY_HIDDEN  __attribute__((visibility("hidden")))
#  define RPY_UNUSED __attribute__ ((__unused__))
#else
#  define RPY_EXPORTED extern __declspec(dllexport)
#  define _RPY_HIDDEN  /* nothing */
#  define RPY_UNUSED   /*nothing */
#endif
#ifndef RPY_EXTERN
#  define RPY_EXTERN   extern _RPY_HIDDEN
#endif

#if defined(_MSC_VER)
  #define INLINE __inline
#elif defined(__GNUC__)
  #define INLINE __inline__
#else
  #error define inline for this compiler
#endif

#endif /* __PYPY_PRECOMMONDEFS_H */

/***** End of precommondefs.h *****/

#include <sys/types.h>
#include <unistd.h>
#include <sys/mman.h>

#include <stdio.h>
#include <stddef.h>   /* for offsetof() */

void dump(char* key, int value) {
    printf("%s: %d\n", key, value);
}


void dump_section_0(void) {
#ifdef MADV_DONTNEED
	dump("defined", 1);
	if ((MADV_DONTNEED) <= 0) {
	    long long x = (long long)(MADV_DONTNEED);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MADV_DONTNEED);
	    printf("value: %llu\n", x);
	}
#else
	dump("defined", 0);
#endif
}

void dump_section_1(void) {
#ifdef MADV_FREE
	dump("defined", 1);
	if ((MADV_FREE) <= 0) {
	    long long x = (long long)(MADV_FREE);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MADV_FREE);
	    printf("value: %llu\n", x);
	}
#else
	dump("defined", 0);
#endif
}

void dump_section_2(void) {
#ifdef MAP_ANON
	dump("defined", 1);
	if ((MAP_ANON) <= 0) {
	    long long x = (long long)(MAP_ANON);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MAP_ANON);
	    printf("value: %llu\n", x);
	}
#else
	dump("defined", 0);
#endif
}

void dump_section_3(void) {
#ifdef MAP_ANONYMOUS
	dump("defined", 1);
	if ((MAP_ANONYMOUS) <= 0) {
	    long long x = (long long)(MAP_ANONYMOUS);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MAP_ANONYMOUS);
	    printf("value: %llu\n", x);
	}
#else
	dump("defined", 0);
#endif
}

void dump_section_4(void) {
#ifdef MAP_DENYWRITE
	dump("defined", 1);
	if ((MAP_DENYWRITE) <= 0) {
	    long long x = (long long)(MAP_DENYWRITE);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MAP_DENYWRITE);
	    printf("value: %llu\n", x);
	}
#else
	dump("defined", 0);
#endif
}

void dump_section_5(void) {
#ifdef MAP_EXECUTABLE
	dump("defined", 1);
	if ((MAP_EXECUTABLE) <= 0) {
	    long long x = (long long)(MAP_EXECUTABLE);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MAP_EXECUTABLE);
	    printf("value: %llu\n", x);
	}
#else
	dump("defined", 0);
#endif
}

void dump_section_6(void) {
	if ((MAP_FIXED) <= 0) {
	    long long x = (long long)(MAP_FIXED);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MAP_FIXED);
	    printf("value: %llu\n", x);
	}
}

void dump_section_7(void) {
#ifdef MAP_NORESERVE
	dump("defined", 1);
	if ((MAP_NORESERVE) <= 0) {
	    long long x = (long long)(MAP_NORESERVE);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MAP_NORESERVE);
	    printf("value: %llu\n", x);
	}
#else
	dump("defined", 0);
#endif
}

void dump_section_8(void) {
	if ((MAP_PRIVATE) <= 0) {
	    long long x = (long long)(MAP_PRIVATE);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MAP_PRIVATE);
	    printf("value: %llu\n", x);
	}
}

void dump_section_9(void) {
	if ((MAP_SHARED) <= 0) {
	    long long x = (long long)(MAP_SHARED);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MAP_SHARED);
	    printf("value: %llu\n", x);
	}
}

void dump_section_10(void) {
#ifdef MREMAP_MAYMOVE
	dump("defined", 1);
	if ((MREMAP_MAYMOVE) <= 0) {
	    long long x = (long long)(MREMAP_MAYMOVE);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MREMAP_MAYMOVE);
	    printf("value: %llu\n", x);
	}
#else
	dump("defined", 0);
#endif
}

void dump_section_11(void) {
	if ((MS_SYNC) <= 0) {
	    long long x = (long long)(MS_SYNC);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(MS_SYNC);
	    printf("value: %llu\n", x);
	}
}

void dump_section_12(void) {
#ifdef PROT_EXEC
	dump("defined", 1);
	if ((PROT_EXEC) <= 0) {
	    long long x = (long long)(PROT_EXEC);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(PROT_EXEC);
	    printf("value: %llu\n", x);
	}
#else
	dump("defined", 0);
#endif
}

void dump_section_13(void) {
	if ((PROT_READ) <= 0) {
	    long long x = (long long)(PROT_READ);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(PROT_READ);
	    printf("value: %llu\n", x);
	}
}

void dump_section_14(void) {
	if ((PROT_WRITE) <= 0) {
	    long long x = (long long)(PROT_WRITE);
	    printf("value: %lld\n", x);
	} else {
	    unsigned long long x = (unsigned long long)(PROT_WRITE);
	    printf("value: %llu\n", x);
	}
}

void dump_section_15(void) {
	typedef off_t platcheck_t;

	platcheck_t x;
	dump("size",  sizeof(platcheck_t));
	x = 0; x = ~x;
	dump("unsigned", x > 0);
}

void dump_section_16(void) {
	typedef size_t platcheck_t;

	platcheck_t x;
	dump("size",  sizeof(platcheck_t));
	x = 0; x = ~x;
	dump("unsigned", x > 0);
}

int main(int argc, char *argv[]) {
	printf("-+- 0\n");
	dump_section_0();
	printf("---\n");
	printf("-+- 1\n");
	dump_section_1();
	printf("---\n");
	printf("-+- 2\n");
	dump_section_2();
	printf("---\n");
	printf("-+- 3\n");
	dump_section_3();
	printf("---\n");
	printf("-+- 4\n");
	dump_section_4();
	printf("---\n");
	printf("-+- 5\n");
	dump_section_5();
	printf("---\n");
	printf("-+- 6\n");
	dump_section_6();
	printf("---\n");
	printf("-+- 7\n");
	dump_section_7();
	printf("---\n");
	printf("-+- 8\n");
	dump_section_8();
	printf("---\n");
	printf("-+- 9\n");
	dump_section_9();
	printf("---\n");
	printf("-+- 10\n");
	dump_section_10();
	printf("---\n");
	printf("-+- 11\n");
	dump_section_11();
	printf("---\n");
	printf("-+- 12\n");
	dump_section_12();
	printf("---\n");
	printf("-+- 13\n");
	dump_section_13();
	printf("---\n");
	printf("-+- 14\n");
	dump_section_14();
	printf("---\n");
	printf("-+- 15\n");
	dump_section_15();
	printf("---\n");
	printf("-+- 16\n");
	dump_section_16();
	printf("---\n");
	return 0;
}
